@@ -10,11 +10,17 @@
 #include <uhdlib/rfnoc/chdr_ctrl_xport.hpp>
 #include <uhdlib/rfnoc/chdr_packet_writer.hpp>
 #include <uhdlib/rfnoc/mgmt_portal.hpp>
-#include <unordered_set>
+#include <uhdlib/utils/narrow.hpp>
 #include <boost/format.hpp>
+#include <chrono>
 #include <cmath>
+#include <map>
 #include <mutex>
 #include <queue>
+#include <string>
+#include <thread>
+#include <tuple>
+#include <vector>
 
 namespace uhd { namespace rfnoc { namespace mgmt {
 
@@ -23,6 +29,7 @@ using namespace transport;
 
 constexpr bool ALLOW_DAISY_CHAINING = true;
 
+// Unused values are left in as comments for reference.
 constexpr uint16_t REG_EPID_SELF               = 0x00; // RW
 constexpr uint16_t REG_RESET_AND_FLUSH         = 0x04; // W
 constexpr uint16_t REG_OSTRM_CTRL_STATUS       = 0x08; // RW
@@ -34,14 +41,14 @@ constexpr uint16_t REG_OSTRM_FC_HEADROOM       = 0x1C; // W
 constexpr uint16_t REG_OSTRM_BUFF_CAP_BYTES_LO = 0x20; // R
 constexpr uint16_t REG_OSTRM_BUFF_CAP_BYTES_HI = 0x24; // R
 constexpr uint16_t REG_OSTRM_BUFF_CAP_PKTS     = 0x28; // R
-constexpr uint16_t REG_OSTRM_SEQ_ERR_CNT       = 0x2C; // R
-constexpr uint16_t REG_OSTRM_DATA_ERR_CNT      = 0x30; // R
-constexpr uint16_t REG_OSTRM_ROUTE_ERR_CNT     = 0x34; // R
+// constexpr uint16_t REG_OSTRM_SEQ_ERR_CNT       = 0x2C; // R
+// constexpr uint16_t REG_OSTRM_DATA_ERR_CNT      = 0x30; // R
+// constexpr uint16_t REG_OSTRM_ROUTE_ERR_CNT     = 0x34; // R
 constexpr uint16_t REG_ISTRM_CTRL_STATUS       = 0x38; // RW
 
 constexpr uint32_t RESET_AND_FLUSH_OSTRM = (1 << 0);
 constexpr uint32_t RESET_AND_FLUSH_ISTRM = (1 << 1);
-constexpr uint32_t RESET_AND_FLUSH_CTRL  = (1 << 2);
+// constexpr uint32_t RESET_AND_FLUSH_CTRL  = (1 << 2);
 constexpr uint32_t RESET_AND_FLUSH_ALL   = 0x7;
 
 #ifdef UHD_BIG_ENDIAN
@@ -67,7 +74,7 @@ constexpr uint32_t STRM_STATUS_SETUP_PENDING = 0x20000000;
 
 
 //! The type of a node in the data-flow graph
-enum node_type {
+enum class node_type {
     //! Invalid type. The FPGA will never have a node with type = 0
     NODE_TYPE_INVALID = 0,
     //! CHDR Crossbar
@@ -84,7 +91,7 @@ struct node_id_t
     //! A unique ID for device that houses this node
     device_id_t device_id = NULL_DEVICE_ID;
     //! The type of this node
-    node_type type = NODE_TYPE_INVALID;
+    node_type type = node_type::NODE_TYPE_INVALID;
     //! The instance number of this node in the device
     sep_inst_t inst = 0;
     //! Extended info about node (not used for comparisons)
@@ -106,7 +113,7 @@ struct node_id_t
     }
     node_id_t(const sep_addr_t& sep_addr)
         : device_id(sep_addr.first)
-        , type(NODE_TYPE_STRM_EP)
+        , type(node_type::NODE_TYPE_STRM_EP)
         , inst(sep_addr.second)
         , extended_info(0)
     {
@@ -120,10 +127,10 @@ struct node_id_t
     inline std::string to_string() const
     {
         static const std::map<node_type, std::string> NODE_STR = {
-            {NODE_TYPE_INVALID, "unknown"},
-            {NODE_TYPE_XBAR, "xbar"},
-            {NODE_TYPE_STRM_EP, "sep"},
-            {NODE_TYPE_XPORT, "xport"}};
+            {node_type::NODE_TYPE_INVALID, "unknown"},
+            {node_type::NODE_TYPE_XBAR, "xbar"},
+            {node_type::NODE_TYPE_STRM_EP, "sep"},
+            {node_type::NODE_TYPE_XPORT, "xport"}};
         return str(
             boost::format("device:%d/%s:%d") % device_id % NODE_STR.at(type) % inst);
     }
@@ -182,10 +189,10 @@ public:
         : _protover(pkt_factory.get_protover())
         , _chdr_w(pkt_factory.get_chdr_w())
         , _endianness(pkt_factory.get_endianness())
-        , _my_node_id(my_sep_addr.first, NODE_TYPE_STRM_EP, xport.get_epid())
+        , _my_node_id(my_sep_addr.first, node_type::NODE_TYPE_STRM_EP, xport.get_epid())
         , _send_seqnum(0)
-        , _send_pkt(std::move(pkt_factory.make_mgmt()))
-        , _recv_pkt(std::move(pkt_factory.make_mgmt()))
+        , _send_pkt(pkt_factory.make_mgmt())
+        , _recv_pkt(pkt_factory.make_mgmt())
     {
         std::lock_guard<std::recursive_mutex> lock(_mutex);
         _discover_topology(xport);
@@ -196,22 +203,22 @@ public:
         }
     }
 
-    virtual ~mgmt_portal_impl() {}
+    ~mgmt_portal_impl() override {}
 
-    virtual const std::set<sep_addr_t>& get_reachable_endpoints() const
+    const std::set<sep_addr_t>& get_reachable_endpoints() const override
     {
         return _discovered_ep_set;
     }
 
-    virtual void initialize_endpoint(
-        chdr_ctrl_xport& xport, const sep_addr_t& addr, const sep_id_t& epid)
+    void initialize_endpoint(
+        chdr_ctrl_xport& xport, const sep_addr_t& addr, const sep_id_t& epid) override
     {
         std::lock_guard<std::recursive_mutex> lock(_mutex);
 
         auto my_epid = xport.get_epid();
 
         // Create a node ID from lookup info
-        node_id_t lookup_node(addr.first, NODE_TYPE_STRM_EP, addr.second);
+        node_id_t lookup_node(addr.first, node_type::NODE_TYPE_STRM_EP, addr.second);
         if (_node_addr_map.count(lookup_node) == 0) {
             throw uhd::lookup_error(
                 "initialize_endpoint(): Cannot reach node with specified address.");
@@ -237,14 +244,14 @@ public:
         register_endpoint(addr, epid);
     }
 
-    virtual void register_endpoint(const sep_addr_t& addr, const sep_id_t& epid)
+    void register_endpoint(const sep_addr_t& addr, const sep_id_t& epid) override
     {
         std::lock_guard<std::recursive_mutex> lock(_mutex);
         if (is_endpoint_registered(epid)) {
             return;
         }
         // Create a node ID from lookup info
-        node_id_t lookup_node(addr.first, NODE_TYPE_STRM_EP, addr.second);
+        node_id_t lookup_node(addr.first, node_type::NODE_TYPE_STRM_EP, addr.second);
         if (_node_addr_map.count(lookup_node) == 0) {
             throw uhd::lookup_error(
                 "initialize_endpoint(): Cannot reach node with specified address.");
@@ -260,13 +267,13 @@ public:
                 % epid % to_string(_node_addr_map.at(lookup_node))));
     }
 
-    virtual bool is_endpoint_registered(const sep_id_t& epid) const
+    bool is_endpoint_registered(const sep_id_t& epid) const override
     {
         std::lock_guard<std::recursive_mutex> lock(_mutex);
         return (_epid_addr_map.count(epid) > 0);
     }
 
-    virtual sep_info_t get_endpoint_info(const sep_id_t& epid) const
+    sep_info_t get_endpoint_info(const sep_id_t& epid) const override
     {
         std::lock_guard<std::recursive_mutex> lock(_mutex);
 
@@ -295,7 +302,7 @@ public:
         return retval;
     }
 
-    virtual void setup_local_route(chdr_ctrl_xport& xport, const sep_id_t& dst_epid)
+    void setup_local_route(chdr_ctrl_xport& xport, const sep_id_t& dst_epid) override
     {
         std::lock_guard<std::recursive_mutex> lock(_mutex);
         auto my_epid = xport.get_epid();
@@ -309,8 +316,9 @@ public:
             mgmt_payload init_req_xact;
             _traverse_to_node(init_req_xact, route_addr);
             _push_node_init_hop(init_req_xact, addr_pair.first, my_epid);
-            const mgmt_payload resp_xact =
-                _send_recv_mgmt_transaction(xport, init_req_xact);
+            // Send the transaction and receive a response.
+            // We don't care about the contents of the response.
+            _send_recv_mgmt_transaction(xport, init_req_xact);
             route_addr.push_back(addr_pair);
         }
 
@@ -324,7 +332,7 @@ public:
             const next_dest_t& curr_dest = addr_pair.second;
             mgmt_hop_t curr_cfg_hop;
             switch (curr_node.type) {
-                case NODE_TYPE_XBAR: {
+                case node_type::NODE_TYPE_XBAR: {
                     // Configure the routing table to route all packets going to dst_epid
                     // to the port with index next_dest_t
                     curr_cfg_hop.add_op(mgmt_op_t(mgmt_op_t::MGMT_OP_CFG_WR_REQ,
@@ -332,7 +340,7 @@ public:
                     curr_cfg_hop.add_op(mgmt_op_t(mgmt_op_t::MGMT_OP_SEL_DEST,
                         mgmt_op_t::sel_dest_payload(static_cast<uint16_t>(curr_dest))));
                 } break;
-                case NODE_TYPE_XPORT: {
+                case node_type::NODE_TYPE_XPORT: {
                     uint8_t node_subtype =
                         static_cast<uint8_t>(curr_node.extended_info & 0xFF);
                     // Run a hop configuration function for custom transports
@@ -345,7 +353,7 @@ public:
                         curr_cfg_hop.add_op(mgmt_op_t(mgmt_op_t::MGMT_OP_NOP));
                     }
                 } break;
-                case NODE_TYPE_STRM_EP: {
+                case node_type::NODE_TYPE_STRM_EP: {
                     // Stream are not involved in routing, so do nothing
                 } break;
                 default: {
@@ -369,7 +377,7 @@ public:
         // Send the transaction and validate that we saw a stream endpoint
         const mgmt_payload sep_info_xact = _send_recv_mgmt_transaction(xport, cfg_xact);
         const node_id_t sep_node         = _pop_node_discovery_hop(sep_info_xact);
-        if (sep_node.type != NODE_TYPE_STRM_EP) {
+        if (sep_node.type != node_type::NODE_TYPE_STRM_EP) {
             throw uhd::routing_error(
                 "Route setup failed. Could not confirm terminal stream endpoint");
         }
@@ -383,8 +391,8 @@ public:
                 % dst_epid % to_string(node_addr)));
     }
 
-    virtual bool can_remote_route(
-        const sep_addr_t& dst_addr, const sep_addr_t& src_addr) const
+    bool can_remote_route(
+        const sep_addr_t& dst_addr, const sep_addr_t& src_addr) const override
     {
         std::lock_guard<std::recursive_mutex> lock(_mutex);
 
@@ -404,7 +412,8 @@ public:
         // Find a common parent (could be faster than n^2 but meh, this is easier)
         for (const auto& dnode : dst_node_addr) {
             for (const auto& snode : src_node_addr) {
-                if (dnode.first == snode.first && dnode.first.type == NODE_TYPE_XBAR) {
+                if (dnode.first == snode.first
+                    && dnode.first.type == node_type::NODE_TYPE_XBAR) {
                     return true;
                 }
             }
@@ -412,8 +421,9 @@ public:
         return false;
     }
 
-    virtual void setup_remote_route(
-        chdr_ctrl_xport& xport, const sep_id_t& dst_epid, const sep_id_t& src_epid)
+    void setup_remote_route(chdr_ctrl_xport& xport,
+        const sep_id_t& dst_epid,
+        const sep_id_t& src_epid) override
     {
         std::lock_guard<std::recursive_mutex> lock(_mutex);
 
@@ -450,14 +460,14 @@ public:
                 % src_epid % dst_epid));
     }
 
-    virtual void config_local_rx_stream_start(chdr_ctrl_xport& xport,
+    void config_local_rx_stream_start(chdr_ctrl_xport& xport,
         const sep_id_t& epid,
         const bool lossy_xport,
         const sw_buff_t pyld_buff_fmt,
         const sw_buff_t mdata_buff_fmt,
         const stream_buff_params_t& fc_freq,
         const stream_buff_params_t& fc_headroom,
-        const bool reset = false)
+        const bool reset = false) override
     {
         std::lock_guard<std::recursive_mutex> lock(_mutex);
         auto my_epid = xport.get_epid();
@@ -501,14 +511,16 @@ public:
             (boost::format("Initiated RX stream setup for EPID=%d") % epid));
     }
 
-    virtual stream_buff_params_t config_local_rx_stream_commit(
-        chdr_ctrl_xport& xport, const sep_id_t& epid, const double timeout = 0.2)
+    stream_buff_params_t config_local_rx_stream_commit(chdr_ctrl_xport& xport,
+        const sep_id_t& epid,
+        const double timeout  = 0.2,
+        const bool fc_enabled = true) override
     {
         std::lock_guard<std::recursive_mutex> lock(_mutex);
 
         // Wait for stream configuration to finish on the HW side
         const node_addr_t& node_addr = _lookup_sep_node_addr(epid);
-        _validate_stream_setup(xport, node_addr, timeout);
+        _validate_stream_setup(xport, node_addr, timeout, fc_enabled);
 
         UHD_LOG_DEBUG("RFNOC::MGMT",
             (boost::format("Finished RX stream setup for EPID=%d") % epid));
@@ -517,11 +529,11 @@ public:
         return std::get<1>(_get_ostrm_status(xport, node_addr));
     }
 
-    virtual void config_local_tx_stream(chdr_ctrl_xport& xport,
+    void config_local_tx_stream(chdr_ctrl_xport& xport,
         const sep_id_t& epid,
         const sw_buff_t pyld_buff_fmt,
         const sw_buff_t mdata_buff_fmt,
-        const bool reset = false)
+        const bool reset = false) override
     {
         std::lock_guard<std::recursive_mutex> lock(_mutex);
         auto my_epid = xport.get_epid();
@@ -561,17 +573,18 @@ public:
             (boost::format("Finished TX stream setup for EPID=%d") % epid));
     }
 
-    virtual stream_buff_params_t config_remote_stream(chdr_ctrl_xport& xport,
+    stream_buff_params_t config_remote_stream(chdr_ctrl_xport& xport,
         const sep_id_t& dst_epid,
         const sep_id_t& src_epid,
         const bool lossy_xport,
         const stream_buff_params_t& fc_freq,
         const stream_buff_params_t& fc_headroom,
         const bool reset     = false,
-        const double timeout = 0.2)
+        const double timeout = 0.2) override
     {
         std::lock_guard<std::recursive_mutex> lock(_mutex);
         auto my_epid = xport.get_epid();
+        const bool fc_enabled = (fc_freq.bytes != 0) || (fc_freq.packets != 0);
 
         // First setup a route between the two endpoints
         setup_remote_route(xport, dst_epid, src_epid);
@@ -617,8 +630,26 @@ public:
             _send_recv_mgmt_transaction(xport, cfg_xact);
         }
 
+        // Build a management transaction to configure the destination node
+        {
+            mgmt_payload cfg_xact;
+            cfg_xact.set_header(my_epid, _protover, _chdr_w);
+            _traverse_to_node(cfg_xact, dst_node_addr);
+            mgmt_hop_t cfg_hop;
+            // Configure buffer types
+            cfg_hop.add_op(mgmt_op_t(mgmt_op_t::MGMT_OP_CFG_WR_REQ,
+                mgmt_op_t::cfg_payload(REG_ISTRM_CTRL_STATUS,
+                    BUILD_CTRL_STATUS_WORD(false, false, BUFF_U64, BUFF_U64, false))));
+            // Return the packet back to us
+            cfg_hop.add_op(mgmt_op_t(mgmt_op_t::MGMT_OP_RETURN));
+            // Send the transaction and receive a response.
+            // We don't care about the contents of the response.
+            cfg_xact.add_hop(cfg_hop);
+            _send_recv_mgmt_transaction(xport, cfg_xact);
+        }
+
         // Wait for stream configuration to finish on the HW side
-        _validate_stream_setup(xport, src_node_addr, timeout);
+        _validate_stream_setup(xport, src_node_addr, timeout, fc_enabled);
 
         UHD_LOG_DEBUG("RFNOC::MGMT",
             (boost::format("Setup a stream from EPID=%d to EPID=%d") % src_epid
@@ -629,9 +660,9 @@ public:
     }
 
 
-    virtual void register_xport_hop_cfg_fns(uint8_t xport_subtype,
+    void register_xport_hop_cfg_fns(uint8_t xport_subtype,
         xport_cfg_fn_t init_hop_cfg_fn,
-        xport_cfg_fn_t rtcfg_hop_cfg_fn)
+        xport_cfg_fn_t rtcfg_hop_cfg_fn) override
     {
         _init_cfg_fns[xport_subtype]  = init_hop_cfg_fn;
         _rtcfg_cfg_fns[xport_subtype] = rtcfg_hop_cfg_fn;
@@ -730,15 +761,16 @@ private: // Functions
                 // Initialize the node (first time config)
                 mgmt_payload init_req_xact(route_xact);
                 _push_node_init_hop(init_req_xact, new_node, my_epid);
-                const mgmt_payload init_resp_xact =
-                    _send_recv_mgmt_transaction(xport, init_req_xact);
+                // Send the transaction and receive a response.
+                // We don't care about the contents of the response.
+                _send_recv_mgmt_transaction(xport, init_req_xact);
                 UHD_LOG_DEBUG("RFNOC::MGMT", "Initialized node " << new_node.to_string());
 
                 // If the new node is a stream endpoint then we are done traversing this
                 // path. If not, then check all ports downstream of the new node and add
                 // them to pending_paths for further traversal
                 switch (new_node.type) {
-                    case NODE_TYPE_XBAR: {
+                    case node_type::NODE_TYPE_XBAR: {
                         // Total ports on this crossbar
                         size_t nports =
                             static_cast<size_t>(new_node.extended_info & 0xFF);
@@ -763,13 +795,13 @@ private: // Functions
                                  << " transports and we are hooked up on port "
                                  << new_node.inst);
                     } break;
-                    case NODE_TYPE_STRM_EP: {
+                    case node_type::NODE_TYPE_STRM_EP: {
                         // Stop searching when we find a stream endpoint
                         // Add the endpoint to the discovered endpoint vector
                         _discovered_ep_set.insert(
                             sep_addr_t(new_node.device_id, new_node.inst));
                     } break;
-                    case NODE_TYPE_XPORT: {
+                    case node_type::NODE_TYPE_XPORT: {
                         // A transport has only one output. We don't need to take
                         // any action to reach
                         pending_paths.push(std::make_pair(new_node, -1));
@@ -789,9 +821,10 @@ private: // Functions
         for (const auto& addr_pair : node_addr) {
             const node_id_t& curr_node   = addr_pair.first;
             const next_dest_t& curr_dest = addr_pair.second;
-            if (curr_node.type != NODE_TYPE_STRM_EP) {
+            if (curr_node.type != node_type::NODE_TYPE_STRM_EP) {
                 // If a node is a crossbar, then it have have a non-negative destination
-                UHD_ASSERT_THROW((curr_node.type != NODE_TYPE_XBAR || curr_dest >= 0));
+                UHD_ASSERT_THROW(
+                    (curr_node.type != node_type::NODE_TYPE_XBAR || curr_dest >= 0));
                 _push_advance_hop(transaction, curr_dest);
             } else {
                 // This is a stream endpoint. Nothing needs to be done to advance
@@ -885,7 +918,7 @@ private: // Functions
             throw uhd::op_failed("Management operation failed. Incorrect format (hops).");
         }
         const mgmt_hop_t& rhop = resp_xact.get_hop(0);
-        if (rhop.get_num_ops() <= 1
+        if (rhop.get_num_ops() < 2
             || rhop.get_op(0).get_op_code() != mgmt_op_t::MGMT_OP_NOP) {
             throw uhd::op_failed(
                 "Management operation failed. Incorrect format (operations).");
@@ -911,17 +944,19 @@ private: // Functions
     }
 
     // Make sure that stream setup is complete and successful, else throw exception
-    void _validate_stream_setup(
-        chdr_ctrl_xport& xport, const node_addr_t& node_addr, const double timeout)
+    void _validate_stream_setup(chdr_ctrl_xport& xport,
+        const node_addr_t& node_addr,
+        const double timeout,
+        const bool fc_enabled)
     {
         // Get the status of the output stream
         uint32_t ostrm_status = 0;
-        double sleep_s        = 0.05;
+        double sleep_s        = 0.001;
         for (size_t i = 0; i < size_t(std::ceil(timeout / sleep_s)); i++) {
             ostrm_status = std::get<0>(_get_ostrm_status(xport, node_addr));
             if ((ostrm_status & STRM_STATUS_SETUP_PENDING) != 0) {
                 // Wait and retry
-                std::chrono::milliseconds(static_cast<int64_t>(sleep_s * 1000));
+                std::this_thread::sleep_for(std::chrono::milliseconds(static_cast<int64_t>(sleep_s * 1000)));
             } else {
                 // Configuration is done
                 break;
@@ -934,7 +969,7 @@ private: // Functions
         if ((ostrm_status & STRM_STATUS_SETUP_ERR) != 0) {
             throw uhd::op_failed("config_stream: Setup failure");
         }
-        if ((ostrm_status & STRM_STATUS_FC_ENABLED) == 0) {
+        if (fc_enabled != bool(ostrm_status & STRM_STATUS_FC_ENABLED)) {
             throw uhd::op_failed("config_stream: Flow control negotiation failed");
         }
     }
@@ -947,9 +982,13 @@ private: // Functions
             throw uhd::op_failed("Management operation failed. Incorrect format (hops).");
         }
         const mgmt_hop_t& rhop     = transaction.get_hop(0);
+        if (rhop.get_num_ops() < 2) {
+            throw uhd::op_failed(
+                "Management operation failed. Incorrect number of operations.");
+        }
         const mgmt_op_t& nop_resp  = rhop.get_op(0);
         const mgmt_op_t& info_resp = rhop.get_op(1);
-        if (rhop.get_num_ops() <= 1 || nop_resp.get_op_code() != mgmt_op_t::MGMT_OP_NOP
+        if (nop_resp.get_op_code() != mgmt_op_t::MGMT_OP_NOP
             || info_resp.get_op_code() != mgmt_op_t::MGMT_OP_INFO_RESP) {
             throw uhd::op_failed(
                 "Management operation failed. Incorrect format (operations).");
@@ -967,18 +1006,18 @@ private: // Functions
     {
         mgmt_hop_t init_hop;
         switch (node.type) {
-            case NODE_TYPE_XBAR: {
+            case node_type::NODE_TYPE_XBAR: {
                 // Configure the routing table to route all packets going to my_epid back
                 // to the port where the packet is entering
                 // The address for the transaction is the EPID and the data is the port #
                 init_hop.add_op(mgmt_op_t(mgmt_op_t::MGMT_OP_CFG_WR_REQ,
                     mgmt_op_t::cfg_payload(my_epid, node.inst)));
             } break;
-            case NODE_TYPE_STRM_EP: {
+            case node_type::NODE_TYPE_STRM_EP: {
                 // Do nothing
                 init_hop.add_op(mgmt_op_t(mgmt_op_t::MGMT_OP_NOP));
             } break;
-            case NODE_TYPE_XPORT: {
+            case node_type::NODE_TYPE_XPORT: {
                 uint8_t node_subtype = static_cast<uint8_t>(node.extended_info & 0xFF);
                 // Run a hop configuration function for custom transports
                 if (_rtcfg_cfg_fns.count(node_subtype)) {
@@ -1019,11 +1058,11 @@ private: // Functions
         chdr_header header;
         header.set_pkt_type(PKT_TYPE_MGMT);
         header.set_num_mdata(0);
-        header.set_seq_num(_send_seqnum++);
-        header.set_length(payload.get_size_bytes() + (chdr_w_to_bits(_chdr_w) / 8));
+        header.set_seq_num(static_cast<uint16_t>(_send_seqnum++));
+        header.set_length(uhd::narrow_cast<uint16_t>(payload.get_size_bytes() + (chdr_w_to_bits(_chdr_w) / 8)));
         header.set_dst_epid(0);
 
-        auto send_buff = xport.get_send_buff(timeout * 1000);
+        auto send_buff = xport.get_send_buff(static_cast<int32_t>(timeout * 1000));
         if (not send_buff) {
             UHD_LOG_ERROR(
                 "RFNOC::MGMT", "Timed out getting send buff for management transaction");
@@ -1049,7 +1088,7 @@ private: // Functions
         // Send the transaction over the wire
         _send_mgmt_transaction(xport, send);
 
-        auto mgmt_buff = xport.get_mgmt_buff(timeout * 1000);
+        auto mgmt_buff = xport.get_mgmt_buff(static_cast<int32_t>(timeout * 1000));
         if (not mgmt_buff) {
             throw uhd::io_error("Timed out getting recv buff for management transaction");
         }
