@@ -77,7 +77,7 @@ size_t _get_chan_from_map(std::unordered_map<size_t, map_type> map, const std::s
         str(boost::format("Invalid daughterboard frontend name: %s") % fe));
 }
 
-constexpr double DEFAULT_RATE = 200e6;
+constexpr double DEFAULT_RATE  = 200e6;
 constexpr char HW_GAIN_STAGE[] = "hw";
 
 } // namespace
@@ -94,9 +94,17 @@ static constexpr uint32_t SR_LEDS      = PERIPH_BASE + 176 * PERIPH_REG_OFFSET;
 static constexpr uint32_t SR_FP_GPIO   = PERIPH_BASE + 184 * PERIPH_REG_OFFSET;
 static constexpr uint32_t SR_DB_GPIO   = PERIPH_BASE + 192 * PERIPH_REG_OFFSET;
 
+// LED bit positions
+// Green LED on TX/RX port (left SMA)
+static constexpr int SR_LED_TXRX_RX = (1 << 0);
+// Red LED on TX/RX port (left SMA)
+static constexpr int SR_LED_TXRX_TX = (1 << 1);
+// Green LED on RX2 port (right SMA)
+static constexpr int SR_LED_RX2_RX = (1 << 2);
+
 static constexpr uint32_t RB_MISC_IO = PERIPH_BASE + 16 * PERIPH_REG_OFFSET;
 static constexpr uint32_t RB_SPI     = PERIPH_BASE + 17 * PERIPH_REG_OFFSET;
-static constexpr uint32_t RB_LEDS    = PERIPH_BASE + 18 * PERIPH_REG_OFFSET;
+// static constexpr uint32_t RB_LEDS    = PERIPH_BASE + 18 * PERIPH_REG_OFFSET;
 static constexpr uint32_t RB_DB_GPIO = PERIPH_BASE + 19 * PERIPH_REG_OFFSET;
 static constexpr uint32_t RB_FP_GPIO = PERIPH_BASE + 20 * PERIPH_REG_OFFSET;
 
@@ -151,15 +159,15 @@ public:
         // FP-GPIO (the gpio_atr_3000 ctor will initialize default values)
         RFNOC_LOG_TRACE("Creating FP-GPIO interface...");
         _fp_gpio = gpio_atr::gpio_atr_3000::make(_wb_iface,
-            x300_regs::SR_FP_GPIO,
-            x300_regs::RB_FP_GPIO,
-            x300_regs::PERIPH_REG_OFFSET);
+            gpio_atr::gpio_atr_offsets::make_default(x300_regs::SR_FP_GPIO,
+                x300_regs::RB_FP_GPIO,
+                x300_regs::PERIPH_REG_OFFSET));
         // Create the GPIO banks and attributes, and populate them with some default
         // values
         // TODO: Do we need this section? Since the _fp_gpio handles state now, we
         // don't need to stash values here. We only need this if we want to set
         // anything to a default value.
-        for (const gpio_atr::gpio_attr_map_t::value_type attr : gpio_atr::gpio_attr_map) {
+        for (const auto& attr : gpio_atr::gpio_attr_map) {
             // TODO: Default values?
             if (attr.first == usrp::gpio_atr::GPIO_SRC) {
                 // Don't set the SRC
@@ -173,10 +181,22 @@ public:
 
         // LEDs are technically valid for both RX and TX, but let's put them
         // here
-        _leds = gpio_atr::gpio_atr_3000::make_write_only(
-            _wb_iface, x300_regs::SR_LEDS, x300_regs::PERIPH_REG_OFFSET);
+        _leds = gpio_atr::gpio_atr_3000::make(_wb_iface,
+            gpio_atr::gpio_atr_offsets::make_write_only(
+                x300_regs::SR_LEDS, x300_regs::PERIPH_REG_OFFSET));
         _leds->set_atr_mode(
             usrp::gpio_atr::MODE_ATR, usrp::gpio_atr::gpio_atr_3000::MASK_SET_ALL);
+        // Set LEDs to default setting: RX2 LED on RX, TX/RX red LED on TX. The
+        // actual setting depends on the antenna choice and is handled in
+        // _update_atr_leds().
+        _leds->set_atr_reg(gpio_atr::ATR_REG_IDLE, 0);
+        _leds->set_atr_reg(gpio_atr::ATR_REG_RX_ONLY, x300_regs::SR_LED_RX2_RX);
+        _leds->set_atr_reg(gpio_atr::ATR_REG_TX_ONLY, x300_regs::SR_LED_TXRX_TX);
+        // We choose to light both LEDs on full duplex, regardless of the
+        // antenna selection, because the single multi-color LED on the TX/RX
+        // side does not provide useful visual feedback by itself.
+        _leds->set_atr_reg(gpio_atr::ATR_REG_FULL_DUPLEX,
+            x300_regs::SR_LED_RX2_RX | x300_regs::SR_LED_TXRX_TX);
         // We always want to initialize at least one frontend core for both TX and RX
         // RX periphs
         for (size_t i = 0; i < std::max<size_t>(get_num_output_ports(), 1); i++) {
@@ -219,7 +239,7 @@ public:
         }
     } /* ctor */
 
-    ~x300_radio_control_impl()
+    ~x300_radio_control_impl() override
     {
         // nop
     }
@@ -227,7 +247,7 @@ public:
     /**************************************************************************
      * Radio API calls
      *************************************************************************/
-    double set_rate(double rate)
+    double set_rate(double rate) override
     {
         // On X3x0, tick rate can't actually be changed at runtime
         const double actual_rate = get_rate();
@@ -239,7 +259,7 @@ public:
         return actual_rate;
     }
 
-    void set_tx_antenna(const std::string& ant, const size_t chan)
+    void set_tx_antenna(const std::string& ant, const size_t chan) override
     {
         // Antenna changes may result in a change of streaming mode for LF/Basic dboards
         if (_basic_lf_tx) {
@@ -256,14 +276,14 @@ public:
             .set(ant);
     }
 
-    std::string get_tx_antenna(const size_t chan) const
+    std::string get_tx_antenna(const size_t chan) const override
     {
         return get_tree()
             ->access<std::string>(get_db_path("tx", chan) / "antenna" / "value")
             .get();
     }
 
-    std::vector<std::string> get_tx_antennas(size_t chan) const
+    std::vector<std::string> get_tx_antennas(size_t chan) const override
     {
         return get_tree()
             ->access<std::vector<std::string>>(
@@ -271,7 +291,7 @@ public:
             .get();
     }
 
-    void set_rx_antenna(const std::string& ant, const size_t chan)
+    void set_rx_antenna(const std::string& ant, const size_t chan) override
     {
         // Antenna changes may result in a change of streaming mode for LF/Basic dboards
         if (_basic_lf_rx) {
@@ -290,14 +310,14 @@ public:
             .set(ant);
     }
 
-    std::string get_rx_antenna(const size_t chan) const
+    std::string get_rx_antenna(const size_t chan) const override
     {
         return get_tree()
             ->access<std::string>(get_db_path("rx", chan) / "antenna" / "value")
             .get();
     }
 
-    std::vector<std::string> get_rx_antennas(size_t chan) const
+    std::vector<std::string> get_rx_antennas(size_t chan) const override
     {
         return get_tree()
             ->access<std::vector<std::string>>(
@@ -305,7 +325,7 @@ public:
             .get();
     }
 
-    double set_tx_frequency(const double freq, const size_t chan)
+    double set_tx_frequency(const double freq, const size_t chan) override
     {
         return get_tree()
             ->access<double>(get_db_path("tx", chan) / "freq" / "value")
@@ -313,7 +333,7 @@ public:
             .get();
     }
 
-    void set_tx_tune_args(const uhd::device_addr_t& tune_args, const size_t chan)
+    void set_tx_tune_args(const uhd::device_addr_t& tune_args, const size_t chan) override
     {
         if (get_tree()->exists(get_db_path("tx", chan) / "tune_args")) {
             get_tree()
@@ -322,14 +342,14 @@ public:
         }
     }
 
-    double get_tx_frequency(const size_t chan)
+    double get_tx_frequency(const size_t chan) override
     {
         return get_tree()
             ->access<double>(get_db_path("tx", chan) / "freq" / "value")
             .get();
     }
 
-    double set_rx_frequency(const double freq, const size_t chan)
+    double set_rx_frequency(const double freq, const size_t chan) override
     {
         RFNOC_LOG_TRACE(
             "set_rx_frequency(freq=" << (freq / 1e6) << " MHz, chan=" << chan << ")");
@@ -339,7 +359,7 @@ public:
             .get();
     }
 
-    void set_rx_tune_args(const uhd::device_addr_t& tune_args, const size_t chan)
+    void set_rx_tune_args(const uhd::device_addr_t& tune_args, const size_t chan) override
     {
         if (get_tree()->exists(get_db_path("rx", chan) / "tune_args")) {
             get_tree()
@@ -348,21 +368,21 @@ public:
         }
     }
 
-    double get_rx_frequency(const size_t chan)
+    double get_rx_frequency(const size_t chan) override
     {
         return get_tree()
             ->access<double>(get_db_path("rx", chan) / "freq" / "value")
             .get();
     }
 
-    uhd::freq_range_t get_tx_frequency_range(const size_t chan) const
+    uhd::freq_range_t get_tx_frequency_range(const size_t chan) const override
     {
         return get_tree()
             ->access<uhd::freq_range_t>(get_db_path("tx", chan) / "freq" / "range")
             .get();
     }
 
-    uhd::freq_range_t get_rx_frequency_range(const size_t chan) const
+    uhd::freq_range_t get_rx_frequency_range(const size_t chan) const override
     {
         return get_tree()
             ->access<uhd::meta_range_t>(get_db_path("rx", chan) / "freq" / "range")
@@ -370,7 +390,7 @@ public:
     }
 
     /*** Bandwidth-Related APIs************************************************/
-    double set_rx_bandwidth(const double bandwidth, const size_t chan)
+    double set_rx_bandwidth(const double bandwidth, const size_t chan) override
     {
         return get_tree()
             ->access<double>(get_db_path("rx", chan) / "bandwidth" / "value")
@@ -378,21 +398,21 @@ public:
             .get();
     }
 
-    double get_rx_bandwidth(const size_t chan)
+    double get_rx_bandwidth(const size_t chan) override
     {
         return get_tree()
             ->access<double>(get_db_path("rx", chan) / "bandwidth" / "value")
             .get();
     }
 
-    uhd::meta_range_t get_rx_bandwidth_range(size_t chan) const
+    uhd::meta_range_t get_rx_bandwidth_range(size_t chan) const override
     {
         return get_tree()
             ->access<uhd::meta_range_t>(get_db_path("rx", chan) / "bandwidth" / "range")
             .get();
     }
 
-    double set_tx_bandwidth(const double bandwidth, const size_t chan)
+    double set_tx_bandwidth(const double bandwidth, const size_t chan) override
     {
         return get_tree()
             ->access<double>(get_db_path("tx", chan) / "bandwidth" / "value")
@@ -400,14 +420,14 @@ public:
             .get();
     }
 
-    double get_tx_bandwidth(const size_t chan)
+    double get_tx_bandwidth(const size_t chan) override
     {
         return get_tree()
             ->access<double>(get_db_path("tx", chan) / "bandwidth" / "value")
             .get();
     }
 
-    uhd::meta_range_t get_tx_bandwidth_range(size_t chan) const
+    uhd::meta_range_t get_tx_bandwidth_range(size_t chan) const override
     {
         return get_tree()
             ->access<uhd::meta_range_t>(get_db_path("tx", chan) / "bandwidth" / "range")
@@ -415,12 +435,13 @@ public:
     }
 
     /*** Gain-Related APIs ***************************************************/
-    double set_tx_gain(const double gain, const size_t chan)
+    double set_tx_gain(const double gain, const size_t chan) override
     {
         return set_tx_gain(gain, ALL_GAINS, chan);
     }
 
-    double set_tx_gain(const double gain, const std::string& name, const size_t chan)
+    double set_tx_gain(
+        const double gain, const std::string& name, const size_t chan) override
     {
         _tx_pwr_mgr.at(chan)->set_tracking_mode(pwr_cal_mgr::tracking_mode::TRACK_GAIN);
         if (_tx_gain_groups.count(chan)) {
@@ -431,12 +452,13 @@ public:
         return radio_control_impl::set_tx_gain(0.0, chan);
     }
 
-    double set_rx_gain(const double gain, const size_t chan)
+    double set_rx_gain(const double gain, const size_t chan) override
     {
         return set_rx_gain(gain, ALL_GAINS, chan);
     }
 
-    double set_rx_gain(const double gain, const std::string& name, const size_t chan)
+    double set_rx_gain(
+        const double gain, const std::string& name, const size_t chan) override
     {
         _rx_pwr_mgr.at(chan)->set_tracking_mode(pwr_cal_mgr::tracking_mode::TRACK_GAIN);
         auto& gg = _rx_gain_groups.at(chan);
@@ -444,42 +466,43 @@ public:
         return radio_control_impl::set_rx_gain(gg->get_value(name), chan);
     }
 
-    double get_rx_gain(const size_t chan)
+    double get_rx_gain(const size_t chan) override
     {
         return get_rx_gain(ALL_GAINS, chan);
     }
 
-    double get_rx_gain(const std::string& name, const size_t chan)
+    double get_rx_gain(const std::string& name, const size_t chan) override
     {
         return _rx_gain_groups.at(chan)->get_value(name);
     }
 
-    double get_tx_gain(const size_t chan)
+    double get_tx_gain(const size_t chan) override
     {
         return get_tx_gain(ALL_GAINS, chan);
     }
 
-    double get_tx_gain(const std::string& name, const size_t chan)
+    double get_tx_gain(const std::string& name, const size_t chan) override
     {
         return _tx_gain_groups.at(chan)->get_value(name);
     }
 
-    std::vector<std::string> get_tx_gain_names(const size_t chan) const
+    std::vector<std::string> get_tx_gain_names(const size_t chan) const override
     {
         return _tx_gain_groups.at(chan)->get_names();
     }
 
-    std::vector<std::string> get_rx_gain_names(const size_t chan) const
+    std::vector<std::string> get_rx_gain_names(const size_t chan) const override
     {
         return _rx_gain_groups.at(chan)->get_names();
     }
 
-    uhd::gain_range_t get_tx_gain_range(const size_t chan) const
+    uhd::gain_range_t get_tx_gain_range(const size_t chan) const override
     {
         return get_tx_gain_range(ALL_GAINS, chan);
     }
 
-    uhd::gain_range_t get_tx_gain_range(const std::string& name, const size_t chan) const
+    uhd::gain_range_t get_tx_gain_range(
+        const std::string& name, const size_t chan) const override
     {
         if (!_tx_gain_groups.count(chan)) {
             throw uhd::index_error(
@@ -488,12 +511,13 @@ public:
         return _tx_gain_groups.at(chan)->get_range(name);
     }
 
-    uhd::gain_range_t get_rx_gain_range(const size_t chan) const
+    uhd::gain_range_t get_rx_gain_range(const size_t chan) const override
     {
         return get_rx_gain_range(ALL_GAINS, chan);
     }
 
-    uhd::gain_range_t get_rx_gain_range(const std::string& name, const size_t chan) const
+    uhd::gain_range_t get_rx_gain_range(
+        const std::string& name, const size_t chan) const override
     {
         if (!_rx_gain_groups.count(chan)) {
             throw uhd::index_error(
@@ -502,7 +526,7 @@ public:
         return _rx_gain_groups.at(chan)->get_range(name);
     }
 
-    std::vector<std::string> get_tx_gain_profile_names(const size_t chan) const
+    std::vector<std::string> get_tx_gain_profile_names(const size_t chan) const override
     {
         return get_tree()
             ->access<std::vector<std::string>>(
@@ -510,7 +534,7 @@ public:
             .get();
     }
 
-    std::vector<std::string> get_rx_gain_profile_names(const size_t chan) const
+    std::vector<std::string> get_rx_gain_profile_names(const size_t chan) const override
     {
         return get_tree()
             ->access<std::vector<std::string>>(
@@ -519,14 +543,14 @@ public:
     }
 
 
-    void set_tx_gain_profile(const std::string& profile, const size_t chan)
+    void set_tx_gain_profile(const std::string& profile, const size_t chan) override
     {
         get_tree()
             ->access<std::string>(get_db_path("tx", chan) / "gains/all/profile/value")
             .set(profile);
     }
 
-    void set_rx_gain_profile(const std::string& profile, const size_t chan)
+    void set_rx_gain_profile(const std::string& profile, const size_t chan) override
     {
         get_tree()
             ->access<std::string>(get_db_path("rx", chan) / "gains/all/profile/value")
@@ -534,14 +558,14 @@ public:
     }
 
 
-    std::string get_tx_gain_profile(const size_t chan) const
+    std::string get_tx_gain_profile(const size_t chan) const override
     {
         return get_tree()
             ->access<std::string>(get_db_path("tx", chan) / "gains/all/profile/value")
             .get();
     }
 
-    std::string get_rx_gain_profile(const size_t chan) const
+    std::string get_rx_gain_profile(const size_t chan) const override
     {
         return get_tree()
             ->access<std::string>(get_db_path("rx", chan) / "gains/all/profile/value")
@@ -551,7 +575,7 @@ public:
     /**************************************************************************
      * LO controls
      *************************************************************************/
-    std::vector<std::string> get_rx_lo_names(const size_t chan) const
+    std::vector<std::string> get_rx_lo_names(const size_t chan) const override
     {
         fs_path rx_fe_fe_root = get_db_path("rx", chan);
         std::vector<std::string> lo_names;
@@ -564,7 +588,7 @@ public:
     }
 
     std::vector<std::string> get_rx_lo_sources(
-        const std::string& name, const size_t chan) const
+        const std::string& name, const size_t chan) const override
     {
         fs_path rx_fe_fe_root = get_db_path("rx", chan);
 
@@ -597,7 +621,7 @@ public:
     }
 
     void set_rx_lo_source(
-        const std::string& src, const std::string& name, const size_t chan)
+        const std::string& src, const std::string& name, const size_t chan) override
     {
         fs_path rx_fe_fe_root = get_db_path("rx", chan);
 
@@ -626,12 +650,15 @@ public:
                 }
             }
         } else {
-            throw uhd::runtime_error(
-                "This device does not support manual configuration of LOs");
+            if (not(src == "internal" and name == ALL_LOS)) {
+                throw uhd::runtime_error(
+                    "This device only supports setting internal source on all LOs");
+            }
         }
     }
 
-    const std::string get_rx_lo_source(const std::string& name, const size_t chan)
+    const std::string get_rx_lo_source(
+        const std::string& name, const size_t chan) override
     {
         fs_path rx_fe_fe_root = get_db_path("rx", chan);
 
@@ -659,7 +686,7 @@ public:
     }
 
     void set_rx_lo_export_enabled(
-        bool enabled, const std::string& name, const size_t chan)
+        bool enabled, const std::string& name, const size_t chan) override
     {
         fs_path rx_fe_fe_root = get_db_path("rx", chan);
 
@@ -686,12 +713,14 @@ public:
                 }
             }
         } else {
-            throw uhd::runtime_error(
-                "This device does not support manual configuration of LOs");
+            if (not(enabled == false and name == ALL_LOS)) {
+                throw uhd::runtime_error("This device only supports setting LO export "
+                                         "enabled to false on all LOs");
+            }
         }
     }
 
-    bool get_rx_lo_export_enabled(const std::string& name, const size_t chan) const
+    bool get_rx_lo_export_enabled(const std::string& name, const size_t chan) override
     {
         fs_path rx_fe_fe_root = get_db_path("rx", chan);
 
@@ -716,7 +745,8 @@ public:
         }
     }
 
-    double set_rx_lo_freq(double freq, const std::string& name, const size_t chan)
+    double set_rx_lo_freq(
+        double freq, const std::string& name, const size_t chan) override
     {
         fs_path rx_fe_fe_root = get_db_path("rx", chan);
 
@@ -742,7 +772,7 @@ public:
         }
     }
 
-    double get_rx_lo_freq(const std::string& name, const size_t chan)
+    double get_rx_lo_freq(const std::string& name, const size_t chan) override
     {
         fs_path rx_fe_fe_root = get_db_path("rx", chan);
 
@@ -765,7 +795,8 @@ public:
         }
     }
 
-    freq_range_t get_rx_lo_freq_range(const std::string& name, const size_t chan) const
+    freq_range_t get_rx_lo_freq_range(
+        const std::string& name, const size_t chan) const override
     {
         fs_path rx_fe_fe_root = get_db_path("rx", chan);
 
@@ -792,7 +823,7 @@ public:
     }
 
     /*** Calibration API *****************************************************/
-    void set_tx_dc_offset(const std::complex<double>& offset, size_t chan)
+    void set_tx_dc_offset(const std::complex<double>& offset, size_t chan) override
     {
         const fs_path dc_offset_path = get_fe_path("tx", chan) / "dc_offset" / "value";
         if (get_tree()->exists(dc_offset_path)) {
@@ -802,7 +833,7 @@ public:
         }
     }
 
-    meta_range_t get_tx_dc_offset_range(size_t chan) const
+    meta_range_t get_tx_dc_offset_range(size_t chan) const override
     {
         const fs_path range_path = get_fe_path("tx", chan) / "dc_offset" / "range";
         if (get_tree()->exists(range_path)) {
@@ -810,11 +841,11 @@ public:
         } else {
             RFNOC_LOG_WARNING(
                 "This device does not support querying the TX DC offset range.");
-            return meta_range_t(0, 0);
+            return meta_range_t(0.0, 0.0);
         }
     }
 
-    void set_tx_iq_balance(const std::complex<double>& correction, size_t chan)
+    void set_tx_iq_balance(const std::complex<double>& correction, size_t chan) override
     {
         const fs_path iq_balance_path = get_fe_path("tx", chan) / "iq_balance" / "value";
         if (get_tree()->exists(iq_balance_path)) {
@@ -824,7 +855,7 @@ public:
         }
     }
 
-    void set_rx_dc_offset(const bool enb, size_t chan)
+    void set_rx_dc_offset(const bool enb, size_t chan) override
     {
         const fs_path dc_offset_path = get_fe_path("rx", chan) / "dc_offset" / "enable";
         if (get_tree()->exists(dc_offset_path)) {
@@ -835,7 +866,7 @@ public:
         }
     }
 
-    void set_rx_dc_offset(const std::complex<double>& offset, size_t chan)
+    void set_rx_dc_offset(const std::complex<double>& offset, size_t chan) override
     {
         const fs_path dc_offset_path = get_fe_path("rx", chan) / "dc_offset" / "value";
         if (get_tree()->exists(dc_offset_path)) {
@@ -845,7 +876,7 @@ public:
         }
     }
 
-    meta_range_t get_rx_dc_offset_range(size_t chan) const
+    meta_range_t get_rx_dc_offset_range(size_t chan) const override
     {
         const fs_path range_path = get_fe_path("rx", chan) / "dc_offset" / "range";
         if (get_tree()->exists(range_path)) {
@@ -853,38 +884,40 @@ public:
         } else {
             RFNOC_LOG_WARNING(
                 "This device does not support querying the rx DC offset range.");
-            return meta_range_t(0, 0);
+            return meta_range_t(0.0, 0.0);
         }
     }
 
-    void set_rx_iq_balance(const bool enb, size_t chan)
+    void set_rx_iq_balance(const bool enb, size_t chan) override
     {
         const fs_path iq_balance_path = get_fe_path("rx", chan) / "iq_balance" / "enable";
         if (get_tree()->exists(iq_balance_path)) {
             get_tree()->access<bool>(iq_balance_path).set(enb);
         } else {
-            RFNOC_LOG_WARNING("Setting RX IQ Balance is not possible on this device.");
+            RFNOC_LOG_WARNING(
+                "Setting automatic RX IQ Balance is not possible on this device.");
         }
     }
 
-    void set_rx_iq_balance(const std::complex<double>& correction, size_t chan)
+    void set_rx_iq_balance(const std::complex<double>& correction, size_t chan) override
     {
         const fs_path iq_balance_path = get_fe_path("rx", chan) / "iq_balance" / "value";
         if (get_tree()->exists(iq_balance_path)) {
             get_tree()->access<std::complex<double>>(iq_balance_path).set(correction);
         } else {
-            RFNOC_LOG_WARNING("Setting RX IQ Balance is not possible on this device.");
+            RFNOC_LOG_WARNING(
+                "Setting manual RX IQ Balance is not possible on this device.");
         }
     }
 
     /*** GPIO API ************************************************************/
-    std::vector<std::string> get_gpio_banks() const
+    std::vector<std::string> get_gpio_banks() const override
     {
         return {"RX", "TX", "FP0"};
     }
 
     void set_gpio_attr(
-        const std::string& bank, const std::string& attr, const uint32_t value)
+        const std::string& bank, const std::string& attr, const uint32_t value) override
     {
         if (bank == "FP0") {
             _fp_gpio->set_gpio_attr(usrp::gpio_atr::gpio_attr_rev_map.at(attr), value);
@@ -894,7 +927,7 @@ public:
             const std::string name          = bank.substr(2);
             const dboard_iface::unit_t unit = (bank[0] == 'R') ? dboard_iface::UNIT_RX
                                                                : dboard_iface::UNIT_TX;
-            constexpr uint16_t mask = 0xFFFF;
+            constexpr uint16_t mask         = 0xFFFF;
             if (attr == "CTRL") {
                 _db_iface->set_pin_ctrl(unit, value, mask);
             } else if (attr == "DDR") {
@@ -921,7 +954,7 @@ public:
             << "'. Ignoring call to set_gpio_attr() to retain backward compatibility.");
     }
 
-    uint32_t get_gpio_attr(const std::string& bank, const std::string& attr)
+    uint32_t get_gpio_attr(const std::string& bank, const std::string& attr) override
     {
         if (bank == "FP0") {
             return _fp_gpio->get_attr_reg(usrp::gpio_atr::gpio_attr_rev_map.at(attr));
@@ -959,7 +992,7 @@ public:
     /**************************************************************************
      * Sensor API
      *************************************************************************/
-    std::vector<std::string> get_rx_sensor_names(size_t chan) const
+    std::vector<std::string> get_rx_sensor_names(size_t chan) const override
     {
         const fs_path sensor_path = get_db_path("rx", chan) / "sensors";
         if (get_tree()->exists(sensor_path)) {
@@ -968,14 +1001,14 @@ public:
         return {};
     }
 
-    uhd::sensor_value_t get_rx_sensor(const std::string& name, size_t chan)
+    uhd::sensor_value_t get_rx_sensor(const std::string& name, size_t chan) override
     {
         return get_tree()
             ->access<uhd::sensor_value_t>(get_db_path("rx", chan) / "sensors" / name)
             .get();
     }
 
-    std::vector<std::string> get_tx_sensor_names(size_t chan) const
+    std::vector<std::string> get_tx_sensor_names(size_t chan) const override
     {
         const fs_path sensor_path = get_db_path("tx", chan) / "sensors";
         if (get_tree()->exists(sensor_path)) {
@@ -984,7 +1017,7 @@ public:
         return {};
     }
 
-    uhd::sensor_value_t get_tx_sensor(const std::string& name, size_t chan)
+    uhd::sensor_value_t get_tx_sensor(const std::string& name, size_t chan) override
     {
         return get_tree()
             ->access<uhd::sensor_value_t>(get_db_path("tx", chan) / "sensors" / name)
@@ -994,7 +1027,7 @@ public:
     /**************************************************************************
      * EEPROM API
      *************************************************************************/
-    void set_db_eeprom(const uhd::eeprom_map_t& db_eeprom)
+    void set_db_eeprom(const uhd::eeprom_map_t& db_eeprom) override
     {
         const std::string key_prefix = db_eeprom.count("rx_id") ? "rx_" : "tx_";
         const std::string id_key     = key_prefix + "id";
@@ -1021,7 +1054,7 @@ public:
     }
 
 
-    uhd::eeprom_map_t get_db_eeprom()
+    uhd::eeprom_map_t get_db_eeprom() override
     {
         uhd::eeprom_map_t result;
         if (get_tree()->exists(DB_PATH / "rx_eeprom")) {
@@ -1044,13 +1077,13 @@ public:
     /**************************************************************************
      * Radio Identification API Calls
      *************************************************************************/
-    std::string get_slot_name() const
+    std::string get_slot_name() const override
     {
         return _radio_type == PRIMARY ? "A" : "B";
     }
 
     size_t get_chan_from_dboard_fe(
-        const std::string& fe, const uhd::direction_t direction) const
+        const std::string& fe, const uhd::direction_t direction) const override
     {
         switch (direction) {
             case uhd::TX_DIRECTION:
@@ -1063,7 +1096,7 @@ public:
     }
 
     std::string get_dboard_fe_from_chan(
-        const size_t chan, const uhd::direction_t direction) const
+        const size_t chan, const uhd::direction_t direction) const override
     {
         switch (direction) {
             case uhd::TX_DIRECTION:
@@ -1075,7 +1108,8 @@ public:
         }
     }
 
-    std::string get_fe_name(const size_t chan, const uhd::direction_t direction) const
+    std::string get_fe_name(
+        const size_t chan, const uhd::direction_t direction) const override
     {
         fs_path name_path =
             get_db_path(direction == uhd::RX_DIRECTION ? "rx" : "tx", chan) / "name";
@@ -1087,7 +1121,7 @@ public:
     }
 
 
-    virtual void set_command_time(uhd::time_spec_t time, const size_t chan)
+    void set_command_time(uhd::time_spec_t time, const size_t chan) override
     {
         node_t::set_command_time(time, chan);
         // This is for TwinRX only:
@@ -1100,30 +1134,31 @@ public:
     /**************************************************************************
      * MB Interface API Calls
      *************************************************************************/
-    uint32_t get_adc_rx_word()
+    uint32_t get_adc_rx_word() override
     {
         return regs().peek32(regmap::RADIO_BASE_ADDR + regmap::REG_RX_DATA);
     }
 
-    void set_adc_test_word(const std::string& patterna, const std::string& patternb)
+    void set_adc_test_word(
+        const std::string& patterna, const std::string& patternb) override
     {
         _adc->set_test_word(patterna, patternb);
     }
 
-    void set_adc_checker_enabled(const bool enb)
+    void set_adc_checker_enabled(const bool enb) override
     {
         _regs->misc_outs_reg.write(
             radio_regmap_t::misc_outs_reg_t::ADC_CHECKER_ENABLED, enb ? 1 : 0);
     }
 
-    bool get_adc_checker_locked(const bool I)
+    bool get_adc_checker_locked(const bool I) override
     {
         return bool(_regs->misc_ins_reg.read(
             I ? radio_regmap_t::misc_ins_reg_t::ADC_CHECKER1_I_LOCKED
               : radio_regmap_t::misc_ins_reg_t::ADC_CHECKER1_Q_LOCKED));
     }
 
-    uint32_t get_adc_checker_error_code(const bool I)
+    uint32_t get_adc_checker_error_code(const bool I) override
     {
         return _regs->misc_ins_reg.get(
             I ? radio_regmap_t::misc_ins_reg_t::ADC_CHECKER1_I_ERROR
@@ -1131,7 +1166,7 @@ public:
     }
 
     // Documented in x300_radio_mbc_iface.hpp
-    void self_test_adc(const uint32_t ramp_time_ms)
+    void self_test_adc(const uint32_t ramp_time_ms) override
     {
         RFNOC_LOG_DEBUG("Running ADC self-cal...");
         // Bypass all front-end corrections
@@ -1208,12 +1243,12 @@ public:
         }
     }
 
-    void sync_dac()
+    void sync_dac() override
     {
         _dac->sync();
     }
 
-    void set_dac_sync(const bool enb, const uhd::time_spec_t& time)
+    void set_dac_sync(const bool enb, const uhd::time_spec_t& time) override
     {
         if (time != uhd::time_spec_t(0.0)) {
             set_command_time(time, 0);
@@ -1225,7 +1260,7 @@ public:
         }
     }
 
-    void dac_verify_sync()
+    void dac_verify_sync() override
     {
         _dac->verify_sync();
     }
@@ -1505,20 +1540,20 @@ private:
         // create a new dboard interface
         x300_dboard_iface_config_t db_config;
         db_config.gpio           = gpio_atr::db_gpio_atr_3000::make(_wb_iface,
-            x300_regs::SR_DB_GPIO,
-            x300_regs::RB_DB_GPIO,
-            x300_regs::PERIPH_REG_OFFSET);
+            gpio_atr::gpio_atr_offsets::make_default(x300_regs::SR_DB_GPIO,
+                x300_regs::RB_DB_GPIO,
+                x300_regs::PERIPH_REG_OFFSET));
         db_config.spi            = _spi;
         db_config.rx_spi_slaveno = DB_RX_SEN;
         db_config.tx_spi_slaveno = DB_TX_SEN;
         db_config.i2c            = zpu_i2c;
         db_config.clock          = clock;
         db_config.which_rx_clk   = (_radio_type == PRIMARY) ? X300_CLOCK_WHICH_DB0_RX
-                                                          : X300_CLOCK_WHICH_DB1_RX;
-        db_config.which_tx_clk = (_radio_type == PRIMARY) ? X300_CLOCK_WHICH_DB0_TX
-                                                          : X300_CLOCK_WHICH_DB1_TX;
-        db_config.dboard_slot   = (_radio_type == PRIMARY) ? 0 : 1;
-        db_config.cmd_time_ctrl = _wb_iface;
+                                                            : X300_CLOCK_WHICH_DB1_RX;
+        db_config.which_tx_clk   = (_radio_type == PRIMARY) ? X300_CLOCK_WHICH_DB0_TX
+                                                            : X300_CLOCK_WHICH_DB1_TX;
+        db_config.dboard_slot    = (_radio_type == PRIMARY) ? 0 : 1;
+        db_config.cmd_time_ctrl  = _wb_iface;
 
         // create a new dboard manager
         RFNOC_LOG_TRACE("Creating DB interface...");
@@ -1558,8 +1593,8 @@ private:
         // here.
         if (num_rx_frontends == 2
             && boost::starts_with(
-                   get_tree()->access<std::string>(DB_PATH / "rx_frontends/0/name").get(),
-                   "TwinRX")) {
+                get_tree()->access<std::string>(DB_PATH / "rx_frontends/0/name").get(),
+                "TwinRX")) {
             _twinrx = true;
             set_num_input_ports(0);
         }
@@ -1752,18 +1787,36 @@ private:
         _db_eeproms[addr] = db_eeprom;
     }
 
+    // A note on updating the LED ATR register: There is a single ATR register
+    // for the radio block, despite there being 2 channels. For most (1-channel)
+    // daughterboards, the rules are simple: When transmitting, the red LED turns
+    // on. When receiving, either the green LED under the RX2 or on the TX/RX
+    // port turn, depending on if the user has selected a TX/RX antenna or not.
+    // For TwinRX, we have additional rules. The board has two channels, but
+    // either of them can used with any SMA port. We therefore have to check
+    // which channels are active (0, 1, or both) and on the active channels,
+    // which set of antenna ports is used (RX1 aka TX/RX, RX2). All active RX
+    // ports shall then be added the the RX ATR register.
     void _update_atr_leds(const std::string& rx_ant, const size_t /*chan*/)
     {
-        // The "RX1" port is used by TwinRX and the "TX/RX" port is used by all
-        // other full-duplex dboards. We need to handle both here.
-        const bool is_txrx = (rx_ant == "TX/RX" or rx_ant == "RX1");
-        const int TXRX_RX  = (1 << 0);
-        const int TXRX_TX  = (1 << 1);
-        const int RX2_RX   = (1 << 2);
-        _leds->set_atr_reg(gpio_atr::ATR_REG_IDLE, 0);
-        _leds->set_atr_reg(gpio_atr::ATR_REG_RX_ONLY, is_txrx ? TXRX_RX : RX2_RX);
-        _leds->set_atr_reg(gpio_atr::ATR_REG_TX_ONLY, TXRX_TX);
-        _leds->set_atr_reg(gpio_atr::ATR_REG_FULL_DUPLEX, RX2_RX | TXRX_TX);
+        uint32_t rx_led_atr_state = 0;
+        if (_twinrx) {
+            for (size_t chan = 0; chan < get_num_output_ports(); chan++) {
+                const auto fe_enable_path = get_db_path("rx", chan) / "enabled";
+                if (get_tree()->access<bool>(fe_enable_path).get()) {
+                    if (get_rx_antenna(chan) == "RX1") {
+                        rx_led_atr_state |= x300_regs::SR_LED_TXRX_RX;
+                    }
+                    if (get_rx_antenna(chan) == "RX2") {
+                        rx_led_atr_state |= x300_regs::SR_LED_RX2_RX;
+                    }
+                }
+            }
+        } else {
+            rx_led_atr_state = rx_ant == "TX/RX" ? x300_regs::SR_LED_TXRX_RX
+                                                 : x300_regs::SR_LED_RX2_RX;
+        }
+        _leds->set_atr_reg(gpio_atr::ATR_REG_RX_ONLY, rx_led_atr_state);
     }
 
     void _set_rx_fe(const std::string& fe, const size_t chan)
@@ -1816,7 +1869,7 @@ private:
     //! Safely shut down all peripherals
     //
     // Reminder: After this is called, no peeks and pokes are allowed!
-    void deinit()
+    void deinit() override
     {
         RFNOC_LOG_TRACE("deinit()");
         // Reset daughterboard
@@ -1840,7 +1893,7 @@ private:
     }
 
     bool check_topology(const std::vector<size_t>& connected_inputs,
-        const std::vector<size_t>& connected_outputs)
+        const std::vector<size_t>& connected_outputs) override
     {
         RFNOC_LOG_TRACE("check_topology()");
         if (!node_t::check_topology(connected_inputs, connected_outputs)) {
@@ -1869,6 +1922,12 @@ private:
                     "Enabling RX chan " << chan << ": " << (chan_active ? "Yes" : "No"));
                 get_tree()->access<bool>(fe_enable_path).set(chan_active);
             }
+            // Modifying the number of active channels can affect how the
+            // front-panel LEDs get configured for TwinRX boards. Worst case,
+            // this is a no-op since we call it with the same argument as it was
+            // called before. Note this must be called after the 'enable'
+            // property is set.
+            _update_atr_leds(get_rx_antenna(chan), chan);
         }
 
         return true;
@@ -1970,7 +2029,7 @@ private:
 
     bool _basic_lf_rx = false;
     bool _basic_lf_tx = false;
-    bool _twinrx = false;
+    bool _twinrx      = false;
 
     std::unordered_map<size_t, rx_fe_perif> _rx_fe_map;
     std::unordered_map<size_t, tx_fe_perif> _tx_fe_map;
